@@ -7,7 +7,13 @@ use indent_write::indentable::Indented;
 use itertools::Itertools;
 use nextest_filtering::errors::FiltersetParseErrors;
 use nextest_metadata::NextestExitCode;
-use nextest_runner::{errors::*, helpers::plural, redact::Redactor};
+use nextest_runner::{
+    errors::*,
+    helpers::{format_interceptor_too_many_tests, plural},
+    list::OwnedTestInstanceId,
+    redact::Redactor,
+    runner::{DebuggerCommand, TracerCommand},
+};
 use owo_colors::OwoColorize;
 use semver::Version;
 use std::{error::Error, io, path::PathBuf, process::ExitStatus, string::FromUtf8Error};
@@ -216,6 +222,34 @@ pub enum ExpectedError {
         /// case)
         is_default: bool,
     },
+    #[error(
+        "--debugger requires exactly one test, but \
+         no tests were selected"
+    )]
+    DebuggerNoTests { debugger: DebuggerCommand },
+    #[error(
+        "--debugger requires exactly one test, but \
+         {test_count} tests were selected"
+    )]
+    DebuggerTooManyTests {
+        debugger: DebuggerCommand,
+        test_count: usize,
+        test_instances: Vec<OwnedTestInstanceId>,
+    },
+    #[error(
+        "--tracer requires exactly one test, but \
+         no tests were selected"
+    )]
+    TracerNoTests { tracer: TracerCommand },
+    #[error(
+        "--tracer requires exactly one test, but \
+         {test_count} tests were selected"
+    )]
+    TracerTooManyTests {
+        tracer: TracerCommand,
+        test_count: usize,
+        test_instances: Vec<OwnedTestInstanceId>,
+    },
     #[cfg(feature = "self-update")]
     #[error("failed to parse --version")]
     UpdateVersionParseError {
@@ -458,6 +492,8 @@ impl ExpectedError {
             Self::SetupScriptFailed => NextestExitCode::SETUP_SCRIPT_FAILED,
             Self::TestRunFailed => NextestExitCode::TEST_RUN_FAILED,
             Self::NoTestsRun { .. } => NextestExitCode::NO_TESTS_RUN,
+            Self::DebuggerNoTests { .. } | Self::DebuggerTooManyTests { .. }
+            | Self::TracerNoTests { .. } | Self::TracerTooManyTests { .. } => NextestExitCode::SETUP_ERROR,
             Self::ArchiveCreateError { .. } => NextestExitCode::ARCHIVE_CREATION_FAILED,
             Self::WriteTestListError { .. }
             | Self::WriteEventError { .. }
@@ -879,6 +915,44 @@ impl ExpectedError {
                 error!("no tests to run{hint_str}");
                 None
             }
+            Self::DebuggerNoTests { debugger: _ } => {
+                error!("--debugger requires exactly one test, but no tests were selected");
+                None
+            }
+            Self::DebuggerTooManyTests {
+                debugger: _,
+                test_count,
+                test_instances,
+            } => {
+                let msg = format_interceptor_too_many_tests(
+                    "debugger",
+                    *test_count,
+                    test_instances,
+                    &styles.list_styles,
+                    styles.bold,
+                );
+                error!("{}", msg);
+                None
+            }
+            Self::TracerNoTests { tracer: _ } => {
+                error!("--tracer requires exactly one test, but no tests were selected");
+                None
+            }
+            Self::TracerTooManyTests {
+                tracer: _,
+                test_count,
+                test_instances,
+            } => {
+                let msg = format_interceptor_too_many_tests(
+                    "tracer",
+                    *test_count,
+                    test_instances,
+                    &styles.list_styles,
+                    styles.bold,
+                );
+                error!("{}", msg);
+                None
+            }
             Self::ShowTestGroupsError { err } => {
                 error!("{err}");
                 err.source()
