@@ -20,10 +20,10 @@
 //!
 //! To solve this issue, we introduce a "cargo-nextest-dup" binary which is exactly the same as
 //! cargo-nextest, except it isn't used as the actual test runner. We refer to it with
-//! `NEXTEST_BIN_EXE_cargo-nextest-dup`.
+//! `NEXTEST_BIN_EXE_cargo_nextest_dup`.
 
 use camino::{Utf8Path, Utf8PathBuf};
-use fixture_data::nextest_tests::EXPECTED_TEST_SUITES;
+use fixture_data::{models::RunProperty, nextest_tests::EXPECTED_TEST_SUITES};
 use integration_tests::{
     env::set_env_vars,
     nextest_cli::{CargoNextestCli, CargoNextestOutput},
@@ -33,6 +33,7 @@ use std::{borrow::Cow, fs::File, io::Write};
 use target_spec::{Platform, summaries::TargetFeaturesSummary};
 
 mod fixtures;
+mod interceptor;
 mod large_alloc;
 mod stuck_signal;
 mod temp_project;
@@ -446,7 +447,7 @@ fn test_run() {
         Some(NextestExitCode::TEST_RUN_FAILED),
         "correct exit code for command\n{output}"
     );
-    check_run_output(&output.stderr, 0);
+    check_run_output_with_junit(&output.stderr, &p.junit_path("default"), 0);
 
     // --exact with nothing else should be the same as above.
     let output = CargoNextestCli::for_test()
@@ -467,7 +468,7 @@ fn test_run() {
         Some(NextestExitCode::TEST_RUN_FAILED),
         "correct exit code for command\n{output}"
     );
-    check_run_output(&output.stderr, 0);
+    check_run_output_with_junit(&output.stderr, &p.junit_path("default"), 0);
 
     // Check the output with --skip.
     let output = CargoNextestCli::for_test()
@@ -488,7 +489,11 @@ fn test_run() {
         Some(NextestExitCode::TEST_RUN_FAILED),
         "correct exit code for command\n{output}"
     );
-    check_run_output(&output.stderr, RunProperty::WithSkipCdylibFilter as u64);
+    check_run_output_with_junit(
+        &output.stderr,
+        &p.junit_path("default"),
+        RunProperty::WithSkipCdylibFilter as u64,
+    );
 
     // Equivalent filterset to the above.
     let output = CargoNextestCli::for_test()
@@ -508,7 +513,11 @@ fn test_run() {
         Some(NextestExitCode::TEST_RUN_FAILED),
         "correct exit code for command\n{output}"
     );
-    check_run_output(&output.stderr, RunProperty::WithSkipCdylibFilter as u64);
+    check_run_output_with_junit(
+        &output.stderr,
+        &p.junit_path("default"),
+        RunProperty::WithSkipCdylibFilter as u64,
+    );
 
     // Check the output with --exact.
     let output = CargoNextestCli::for_test()
@@ -525,8 +534,9 @@ fn test_run() {
         ])
         // The above tests pass so don't pass in unchecked(true) here.
         .output();
-    check_run_output(
+    check_run_output_with_junit(
         &output.stderr,
+        &p.junit_path("default"),
         RunProperty::WithMultiplyTwoExactFilter as u64,
     );
 
@@ -543,8 +553,9 @@ fn test_run() {
         ])
         // The above tests pass so don't pass in unchecked(true) here.
         .output();
-    check_run_output(
+    check_run_output_with_junit(
         &output.stderr,
+        &p.junit_path("default"),
         RunProperty::WithMultiplyTwoExactFilter as u64,
     );
 
@@ -567,8 +578,9 @@ fn test_run() {
         // This should only select the one test_multiply_two test, which pass. So don't pass in
         // unchecked(true) here.
         .output();
-    check_run_output(
+    check_run_output_with_junit(
         &output.stderr,
+        &p.junit_path("default"),
         RunProperty::WithSkipCdylibFilter as u64 | RunProperty::WithMultiplyTwoExactFilter as u64,
     );
 
@@ -586,8 +598,9 @@ fn test_run() {
         // This should only select the test_multiply_two test, which passes. So don't pass in
         // unchecked(true) here.
         .output();
-    check_run_output(
+    check_run_output_with_junit(
         &output.stderr,
+        &p.junit_path("default"),
         RunProperty::WithSkipCdylibFilter as u64 | RunProperty::WithMultiplyTwoExactFilter as u64,
     );
 
@@ -608,8 +621,9 @@ fn test_run() {
         // This should only select the test_multiply_two test, which passes. So don't pass in
         // unchecked(true) here.
         .output();
-    check_run_output(
+    check_run_output_with_junit(
         &output.stderr,
+        &p.junit_path("default"),
         RunProperty::WithSkipCdylibFilter as u64 | RunProperty::WithMultiplyTwoExactFilter as u64,
     );
 
@@ -631,8 +645,9 @@ fn test_run() {
         // This should only select the test_multiply_two test, which passes. So don't pass in
         // unchecked(true) here.
         .output();
-    check_run_output(
+    check_run_output_with_junit(
         &output.stderr,
+        &p.junit_path("default"),
         RunProperty::WithSkipCdylibFilter as u64 | RunProperty::WithMultiplyTwoExactFilter as u64,
     );
 }
@@ -660,7 +675,7 @@ fn test_run_after_build() {
         Some(NextestExitCode::TEST_RUN_FAILED),
         "correct exit code for command\n{output}"
     );
-    check_run_output(&output.stderr, 0);
+    check_run_output_with_junit(&output.stderr, &p.junit_path("default"), 0);
 }
 
 #[test]
@@ -719,7 +734,11 @@ fn test_relocated_run() {
         Some(NextestExitCode::TEST_RUN_FAILED),
         "correct exit code for command\n{output}"
     );
-    check_run_output(&output.stderr, RunProperty::Relocated as u64);
+    check_run_output_with_junit(
+        &output.stderr,
+        &p2.junit_path("default"),
+        RunProperty::Relocated as u64,
+    );
 }
 
 #[test]
@@ -918,7 +937,7 @@ fn test_archive_with_build_filter() {
         );
         run_archive_with_args(
             &archive_file,
-            RunProperty::CdyLibPackageFilter as u64 | RunProperty::SkipSummaryCheck as u64,
+            RunProperty::CdyLibExamplePackageFilter as u64 | RunProperty::SkipSummaryCheck as u64,
             NextestExitCode::OK,
         );
     });
@@ -1113,7 +1132,7 @@ fn run_archive_with_args(
         Some(expected_exit_code),
         "correct exit code for command\n{output}"
     );
-    check_run_output(&output.stderr, run_property);
+    check_run_output_with_junit(&output.stderr, &p2.junit_path("default"), run_property);
 
     // project is included in return value to keep tempdirs alive
     (p2, extract_to.join("target"))
@@ -1414,7 +1433,11 @@ fn test_run_with_default_filter() {
         Some(NextestExitCode::TEST_RUN_FAILED),
         "correct exit code for command\n{output}"
     );
-    check_run_output(&output.stderr, RunProperty::WithDefaultFilter as u64);
+    check_run_output_with_junit(
+        &output.stderr,
+        &p.junit_path("with-default-filter"),
+        RunProperty::WithDefaultFilter as u64,
+    );
 }
 
 #[test]
@@ -1658,7 +1681,7 @@ fn test_rustc_version_verbose_errors() {
     set_env_vars();
 
     // Set RUSTC to the shim.
-    let shim_rustc = std::env::var("NEXTEST_BIN_EXE_rustc-shim").unwrap();
+    let shim_rustc = std::env::var("NEXTEST_BIN_EXE_rustc_shim").unwrap();
 
     let mut command = CargoNextestCli::for_test();
     command
