@@ -3,18 +3,87 @@
 
 use crate::{
     cargo_config::{CargoConfigs, DiscoveredConfig},
-    reporter::{displayer::formatters::DisplayBracketedHhMmSs, events::*, helpers::Styles},
+    helpers::{DisplayTestInstance, plural},
+    list::TestInstanceId,
+    reporter::{
+        displayer::formatters::DisplayBracketedHhMmSs,
+        events::*,
+        helpers::{Styles, print_lines_in_chunks},
+    },
 };
 use indicatif::{ProgressBar, ProgressDrawTarget, ProgressStyle};
+use nextest_metadata::RustBinaryId;
 use owo_colors::OwoColorize;
 use std::{
+    cmp::{max, min},
     env, fmt,
-    io::{self, IsTerminal, Write},
-    time::Duration,
+    io::IsTerminal,
+    str::FromStr,
+    time::{Duration, Instant},
 };
 use swrite::{SWrite, swrite};
 use tracing::debug;
 
+/// The refresh rate for the progress bar, set to a minimal value.
+///
+/// For progress, during each tick, two things happen:
+///
+/// - We update the message, calling self.bar.set_message.
+/// - We print any buffered output.
+///
+/// We want both of these updates to be combined into one terminal flush, so we
+/// set *this* to a minimal value (so self.bar.set_message doesn't do a redraw),
+/// and rely on ProgressBar::print_and_flush_buffer to always flush the
+/// terminal.
+const PROGRESS_REFRESH_RATE_HZ: u8 = 1;
+
+/// The maximum number of running tests to display with
+/// `--show-progress=running` or `only`.
+#[derive(Clone, Copy, Debug, Eq, PartialEq)]
+pub enum MaxProgressRunning {
+    /// Show a specific maximum number of running tests.
+    /// If 0, running tests (including the overflow summary) aren't displayed.
+    Count(usize),
+
+    /// Show all running tests (no limit).
+    Infinite,
+}
+
+impl MaxProgressRunning {
+    /// The default value (8 tests).
+    pub const DEFAULT_VALUE: Self = Self::Count(8);
+}
+
+impl Default for MaxProgressRunning {
+    fn default() -> Self {
+        Self::DEFAULT_VALUE
+    }
+}
+
+impl FromStr for MaxProgressRunning {
+    type Err = String;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        if s.eq_ignore_ascii_case("infinite") {
+            return Ok(Self::Infinite);
+        }
+
+        match s.parse::<usize>() {
+            Err(e) => Err(format!("Error: {e} parsing {s}")),
+            Ok(n) => Ok(Self::Count(n)),
+        }
+    }
+}
+
+impl fmt::Display for MaxProgressRunning {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            Self::Infinite => write!(f, "infinite"),
+            Self::Count(n) => write!(f, "{n}"),
+        }
+    }
+}
+
 /// How to show progress.
 #[derive(Default, Clone, Copy, Debug)]
 pub enum ShowProgress {
@@ -25,16 +94,83 @@ pub enum ShowProgress {
     /// No progress display.
     None,
 
-    /// Show a progress bar.
-    Bar,
-
     /// Show a counter on each line.
     Counter,
+
+    /// Show a progress bar and the running tests
+    Running,
+}
+
+#[derive(Debug)]
+pub(super) enum RunningTestStatus {
+    Running,
+    Slow,
+    Delay(Duration),
+    Retry,
+}
+
+#[derive(Debug)]
+pub(super) struct RunningTest {
+    binary_id: RustBinaryId,
+    test_name: String,
+    status: RunningTestStatus,
+    start_time: Instant,
+    paused_for: Duration,
+}
+
+impl RunningTest {
+    fn message(&self, now: Instant, width: usize, styles: &Styles) -> String {
+        let mut elapsed = (now - self.start_time).saturating_sub(self.paused_for);
+        let status = match self.status {
+            RunningTestStatus::Running => "     ".to_owned(),
+            RunningTestStatus::Slow => " SLOW".style(styles.skip).to_string(),
+            RunningTestStatus::Delay(d) => {
+                // The elapsed might be greater than the delay duration in case
+                // we ticked past the delay duration without receiving a
+                // notification that the test retry started.
+                elapsed = d.saturating_sub(elapsed);
+                "DELAY".style(styles.retry).to_string()
+            }
+            RunningTestStatus::Retry => "RETRY".style(styles.retry).to_string(),
+        };
+        let elapsed = format!(
+            "{:0>2}:{:0>2}:{:0>2}",
+            elapsed.as_secs() / 3600,
+            elapsed.as_secs() / 60,
+            elapsed.as_secs() % 60,
+        );
+        let max_width = width.saturating_sub(25);
+        let test = DisplayTestInstance::new(
+            None,
+            None,
+            TestInstanceId {
+                binary_id: &self.binary_id,
+
+                test_name: &self.test_name,
+            },
+            &styles.list_styles,
+        )
+        .with_max_width(max_width);
+        format!("       {} [{:>9}] {}", status, elapsed, test)
+    }
 }
 
 #[derive(Debug)]
 pub(super) struct ProgressBarState {
     bar: ProgressBar,
+    stats: RunStats,
+    running: usize,
+    max_progress_running: MaxProgressRunning,
+    // Keep track of the maximum number of lines used. This allows to adapt the
+    // size of the 'viewport' to what we are using, and not just to the maximum
+    // number of tests that can be run in parallel
+    max_running_displayed: usize,
+    // None when the running tests are not displayed
+    running_tests: Option<Vec<RunningTest>>,
+    buffer: String,
+    // Size in bytes for chunking println calls. Configurable via the
+    // undocumented __NEXTEST_PROGRESS_PRINTLN_CHUNK_SIZE env var.
+    println_chunk_size: usize,
     // Reasons for hiding the progress bar. We show the progress bar if none of
     // these are set and hide it if any of them are set.
     //
@@ -47,13 +183,15 @@ pub(super) struct ProgressBarState {
     hidden_no_capture: bool,
     hidden_run_paused: bool,
     hidden_info_response: bool,
-    hidden_between_sub_runs: bool,
 }
 
 impl ProgressBarState {
-    pub(super) fn new(test_count: usize, progress_chars: &str) -> Self {
+    pub(super) fn new(
+        test_count: usize,
+        progress_chars: &str,
+        max_progress_running: MaxProgressRunning,
+    ) -> Self {
         let bar = ProgressBar::new(test_count as u64);
-
         let test_count_width = format!("{test_count}").len();
         // Create the template using the width as input. This is a
         // little confusing -- {{foo}} is what's passed into the
@@ -61,7 +199,7 @@ impl ProgressBarState {
         // statement.
         let template = format!(
             "{{prefix:>12}} [{{elapsed_precise:>9}}] {{wide_bar}} \
-            {{pos:>{test_count_width}}}/{{len:{test_count_width}}}: {{msg}}     "
+            {{pos:>{test_count_width}}}/{{len:{test_count_width}}}: {{msg}}"
         );
         bar.set_style(
             ProgressStyle::default_bar()
@@ -70,66 +208,210 @@ impl ProgressBarState {
                 .expect("template is known to be valid"),
         );
 
-        // NOTE: set_draw_target must be called before enable_steady_tick to avoid a
-        // spurious extra line from being printed as the draw target changes.
-        bar.set_draw_target(Self::stderr_target());
-        // Enable a steady tick 10 times a second.
-        bar.enable_steady_tick(Duration::from_millis(100));
+        let running_tests =
+            (!matches!(max_progress_running, MaxProgressRunning::Count(0))).then(Vec::new);
+
+        // The println chunk size defaults to a value chosen by experimentation,
+        // locally and over SSH. This controls how often the progress bar
+        // refreshes during large output bursts.
+        let println_chunk_size = env::var("__NEXTEST_PROGRESS_PRINTLN_CHUNK_SIZE")
+            .ok()
+            .and_then(|s| s.parse::<usize>().ok())
+            .unwrap_or(4096);
 
         Self {
             bar,
+            stats: RunStats::default(),
+            running: 0,
+            max_progress_running,
+            max_running_displayed: 0,
+            running_tests,
+            buffer: String::new(),
+            println_chunk_size,
             hidden_no_capture: false,
             hidden_run_paused: false,
             hidden_info_response: false,
-            hidden_between_sub_runs: false,
         }
     }
 
+    pub(super) fn tick(&mut self, styles: &Styles) {
+        self.update_message(styles);
+        self.print_and_clear_buffer();
+    }
+
+    fn print_and_clear_buffer(&mut self) {
+        self.print_and_force_redraw();
+        self.buffer.clear();
+    }
+
+    /// Prints the contents of the buffer, and always forces a redraw.
+    fn print_and_force_redraw(&self) {
+        if self.buffer.is_empty() {
+            // Force a redraw as part of our contract. See the documentation for
+            // `PROGRESS_REFRESH_RATE_HZ`.
+            self.bar.force_draw();
+            return;
+        }
+
+        // println below also forces a redraw, so we don't need to call
+        // force_draw in this case.
+
+        // ProgressBar::println is only called if there's something in the
+        // buffer, for two reasons:
+        //
+        // 1. If passed in nothing at all, it prints an empty line.
+        // 2. It forces a full redraw.
+        //
+        // But if self.buffer is too large, we can overwhelm the terminal with
+        // large amounts of non-progress-bar output, causing the progress bar to
+        // flicker in and out. To avoid those issues, we chunk the output to
+        // maintain progress bar visibility by redrawing it regularly.
+        print_lines_in_chunks(&self.buffer, self.println_chunk_size, |chunk| {
+            self.bar.println(chunk);
+        });
+    }
+
+    fn update_message(&mut self, styles: &Styles) {
+        let mut msg = progress_bar_msg(&self.stats, self.running, styles);
+        msg += "     ";
+
+        if let Some(running_tests) = &self.running_tests {
+            let (_, width) = console::Term::stderr().size();
+            let width = max(width as usize, 40);
+            let now = Instant::now();
+            let mut count = match self.max_progress_running {
+                MaxProgressRunning::Count(count) => min(running_tests.len(), count),
+                MaxProgressRunning::Infinite => running_tests.len(),
+            };
+            for running_test in &running_tests[..count] {
+                msg.push('\n');
+                msg.push_str(&running_test.message(now, width, styles));
+            }
+            if count < running_tests.len() {
+                let overflow_count = running_tests.len() - count;
+                msg.push_str(&format!(
+                    "\n             ... and {} more {} running",
+                    overflow_count.style(styles.count),
+                    plural::tests_str(overflow_count),
+                ));
+                count += 1;
+            }
+            self.max_running_displayed = max(self.max_running_displayed, count);
+            msg.push_str(&"\n".to_string().repeat(self.max_running_displayed - count));
+        }
+        self.bar.set_message(msg);
+    }
+
     pub(super) fn update_progress_bar(&mut self, event: &TestEvent<'_>, styles: &Styles) {
         let before_should_hide = self.should_hide();
 
         match &event.kind {
+            TestEventKind::StressSubRunStarted { .. } => {
+                self.bar.reset();
+            }
             TestEventKind::StressSubRunFinished { .. } => {
-                // Hide the progress bar between sub runs to avoid a spurious
-                // progress bar.
-                self.hidden_between_sub_runs = true;
+                // Clear all test bars to remove empty lines of output between
+                // sub-runs.
+                self.bar.finish_and_clear();
             }
             TestEventKind::SetupScriptStarted { no_capture, .. } => {
                 // Hide the progress bar if either stderr or stdout are being passed through.
                 if *no_capture {
                     self.hidden_no_capture = true;
                 }
-                self.hidden_between_sub_runs = false;
             }
             TestEventKind::SetupScriptFinished { no_capture, .. } => {
                 // Restore the progress bar if it was hidden.
                 if *no_capture {
                     self.hidden_no_capture = false;
                 }
-                self.hidden_between_sub_runs = false;
             }
             TestEventKind::TestStarted {
                 current_stats,
                 running,
-                ..
-            }
-            | TestEventKind::TestFinished {
-                current_stats,
-                running,
+                test_instance,
                 ..
             } => {
-                self.hidden_between_sub_runs = false;
+                self.running = *running;
+
                 self.bar.set_prefix(progress_bar_prefix(
                     current_stats,
                     current_stats.cancel_reason,
                     styles,
                 ));
-                self.bar
-                    .set_message(progress_bar_msg(current_stats, *running, styles));
                 // If there are skipped tests, the initial run count will be lower than when constructed
                 // in ProgressBar::new.
                 self.bar.set_length(current_stats.initial_run_count as u64);
                 self.bar.set_position(current_stats.finished_count as u64);
+
+                if let Some(running_tests) = &mut self.running_tests {
+                    running_tests.push(RunningTest {
+                        binary_id: test_instance.id().binary_id.clone(),
+                        test_name: test_instance.id().test_name.to_owned(),
+                        status: RunningTestStatus::Running,
+                        start_time: Instant::now(),
+                        paused_for: Duration::ZERO,
+                    });
+                }
+            }
+            TestEventKind::TestFinished {
+                current_stats,
+                running,
+                test_instance,
+                ..
+            } => {
+                self.running = *running;
+                self.remove_test(&test_instance.id());
+
+                self.bar.set_prefix(progress_bar_prefix(
+                    current_stats,
+                    current_stats.cancel_reason,
+                    styles,
+                ));
+                // If there are skipped tests, the initial run count will be lower than when constructed
+                // in ProgressBar::new.
+                self.bar.set_length(current_stats.initial_run_count as u64);
+                self.bar.set_position(current_stats.finished_count as u64);
+            }
+            TestEventKind::TestAttemptFailedWillRetry {
+                test_instance,
+                delay_before_next_attempt,
+                ..
+            } => {
+                self.remove_test(&test_instance.id());
+                if let Some(running_tests) = &mut self.running_tests {
+                    running_tests.push(RunningTest {
+                        binary_id: test_instance.id().binary_id.clone(),
+                        test_name: test_instance.id().test_name.to_owned(),
+                        status: RunningTestStatus::Delay(*delay_before_next_attempt),
+                        start_time: Instant::now(),
+                        paused_for: Duration::ZERO,
+                    });
+                }
+            }
+            TestEventKind::TestRetryStarted { test_instance, .. } => {
+                self.remove_test(&test_instance.id());
+                if let Some(running_tests) = &mut self.running_tests {
+                    running_tests.push(RunningTest {
+                        binary_id: test_instance.id().binary_id.clone(),
+                        test_name: test_instance.id().test_name.to_owned(),
+                        status: RunningTestStatus::Retry,
+                        start_time: Instant::now(),
+                        paused_for: Duration::ZERO,
+                    });
+                }
+            }
+            TestEventKind::TestSlow { test_instance, .. } => {
+                if let Some(running_tests) = &mut self.running_tests {
+                    running_tests
+                        .iter_mut()
+                        .find(|rt| {
+                            &rt.binary_id == test_instance.id().binary_id
+                                && rt.test_name == test_instance.id().test_name
+                        })
+                        .expect("a slow test to be already running")
+                        .status = RunningTestStatus::Slow;
+                }
             }
             TestEventKind::InfoStarted { .. } => {
                 // While info is being displayed, hide the progress bar to avoid
@@ -149,9 +431,15 @@ impl ProgressBarState {
                 // Continuing the run should show the progress bar since we'll
                 // continue to output to it.
                 self.hidden_run_paused = false;
-                // Wish a mutable form of with_elapsed were supported.
-                let bar = std::mem::replace(&mut self.bar, ProgressBar::hidden());
-                self.bar = bar.with_elapsed(event.elapsed);
+                let current_global_elapsed = self.bar.elapsed();
+                self.bar.set_elapsed(event.elapsed);
+
+                if let Some(running_tests) = &mut self.running_tests {
+                    let delta = current_global_elapsed.saturating_sub(event.elapsed);
+                    for running_test in running_tests {
+                        running_test.paused_for += delta;
+                    }
+                }
             }
             TestEventKind::RunBeginCancel { current_stats, .. }
             | TestEventKind::RunBeginKill { current_stats, .. } => {
@@ -172,22 +460,32 @@ impl ProgressBarState {
         }
     }
 
-    pub(super) fn write_buf(&self, buf: &[u8]) -> io::Result<()> {
-        // ProgressBar::println doesn't print status lines if the bar is
-        // hidden. The suspend method prints it in all cases.
-        self.bar.suspend(|| std::io::stderr().write_all(buf))
+    fn remove_test(&mut self, test_instance: &TestInstanceId) {
+        if let Some(running_tests) = &mut self.running_tests {
+            running_tests.remove(
+                running_tests
+                    .iter()
+                    .position(|e| {
+                        &e.binary_id == test_instance.binary_id
+                            && e.test_name == test_instance.test_name
+                    })
+                    .expect("finished test to have started"),
+            );
+        }
+    }
+
+    pub(super) fn write_buf(&mut self, buf: &str) {
+        self.buffer.push_str(buf);
     }
 
     #[inline]
     pub(super) fn finish_and_clear(&self) {
+        self.print_and_force_redraw();
         self.bar.finish_and_clear();
     }
 
     fn stderr_target() -> ProgressDrawTarget {
-        // This used to be unbuffered, but that option went away from indicatif
-        // 0.17.0. The refresh rate is now 20hz so that it's double the steady
-        // tick rate.
-        ProgressDrawTarget::stderr_with_hz(20)
+        ProgressDrawTarget::stderr_with_hz(PROGRESS_REFRESH_RATE_HZ)
     }
 
     fn hidden_target() -> ProgressDrawTarget {
@@ -195,10 +493,7 @@ impl ProgressBarState {
     }
 
     fn should_hide(&self) -> bool {
-        self.hidden_no_capture
-            || self.hidden_run_paused
-            || self.hidden_info_response
-            || self.hidden_between_sub_runs
+        self.hidden_no_capture || self.hidden_run_paused || self.hidden_info_response
     }
 
     pub(super) fn is_hidden(&self) -> bool {
@@ -207,7 +502,10 @@ impl ProgressBarState {
 }
 
 /// OSC 9 terminal progress reporting.
-pub(super) struct TerminalProgress {}
+#[derive(Default)]
+pub(super) struct TerminalProgress {
+    last_value: TerminalProgressValue,
+}
 
 impl TerminalProgress {
     const ENV: &str = "CARGO_TERM_PROGRESS_TERM_INTEGRATION";
@@ -220,7 +518,7 @@ impl TerminalProgress {
                     if let Some(v) = config.term.progress.term_integration {
                         if v {
                             debug!("enabling terminal progress reporting based on {source:?}");
-                            return Some(Self {});
+                            return Some(Self::default());
                         } else {
                             debug!("disabling terminal progress reporting based on {source:?}");
                             return None;
@@ -234,7 +532,7 @@ impl TerminalProgress {
                                 "enabling terminal progress reporting based on \
                                  CARGO_TERM_PROGRESS_TERM_INTEGRATION environment variable"
                             );
-                            return Some(Self {});
+                            return Some(Self::default());
                         } else if v == "false" {
                             debug!(
                                 "disabling terminal progress reporting based on \
@@ -253,7 +551,7 @@ impl TerminalProgress {
                     if let Some(v) = config.term.progress.term_integration {
                         if v {
                             debug!("enabling terminal progress reporting based on {source:?}");
-                            return Some(Self {});
+                            return Some(Self::default());
                         } else {
                             debug!("disabling terminal progress reporting based on {source:?}");
                             return None;
@@ -263,14 +561,10 @@ impl TerminalProgress {
             }
         }
 
-        supports_osc_9_4(stream).then_some(TerminalProgress {})
+        supports_osc_9_4(stream).then(Self::default)
     }
 
-    pub(super) fn update_progress(
-        &self,
-        event: &TestEvent<'_>,
-        writer: &mut dyn Write,
-    ) -> Result<(), io::Error> {
+    pub(super) fn update_progress(&mut self, event: &TestEvent<'_>) {
         let value = match &event.kind {
             TestEventKind::RunStarted { .. }
             | TestEventKind::StressSubRunStarted { .. }
@@ -314,7 +608,11 @@ impl TerminalProgress {
             }
         };
 
-        write!(writer, "{value}")
+        self.last_value = value;
+    }
+
+    pub(super) fn last_value(&self) -> &TerminalProgressValue {
+        &self.last_value
     }
 }
 
@@ -327,16 +625,18 @@ fn supports_osc_9_4(stream: &dyn IsTerminal) -> bool {
         );
         return false;
     }
-    if std::env::var("WT_SESSION").is_ok() {
+    if std::env::var_os("WT_SESSION").is_some() {
         debug!("autodetect terminal progress reporting: enabling since WT_SESSION is set");
         return true;
     };
-    if std::env::var("ConEmuANSI").ok() == Some("ON".into()) {
+    if std::env::var_os("ConEmuANSI").is_some_and(|term| term == "ON") {
         debug!("autodetect terminal progress reporting: enabling since ConEmuANSI is ON");
         return true;
     }
-    if std::env::var("TERM_PROGRAM").ok() == Some("WezTerm".into()) {
-        debug!("autodetect terminal progress reporting: enabling since TERM_PROGRAM is WezTerm");
+    if let Ok(term) = std::env::var("TERM_PROGRAM")
+        && (term == "WezTerm" || term == "ghostty")
+    {
+        debug!("autodetect terminal progress reporting: enabling since TERM_PROGRAM is {term}");
         return true;
     }
 
@@ -346,9 +646,10 @@ fn supports_osc_9_4(stream: &dyn IsTerminal) -> bool {
 /// A progress status value printable as an ANSI OSC 9;4 escape code.
 ///
 /// Adapted from Cargo 1.87.
-#[derive(PartialEq, Debug)]
-enum TerminalProgressValue {
+#[derive(PartialEq, Debug, Default)]
+pub(super) enum TerminalProgressValue {
     /// No output.
+    #[default]
     None,
     /// Remove progress.
     Remove,
@@ -514,6 +815,7 @@ fn progress_bar_cancel_prefix(reason: Option<CancelReason>, styles: &Styles) ->
         | Some(CancelReason::TestFailure)
         | Some(CancelReason::ReportError)
         | Some(CancelReason::GlobalTimeout)
+        | Some(CancelReason::TestFailureImmediate)
         | Some(CancelReason::Signal)
         | Some(CancelReason::Interrupt)
         | None => "Cancelling",
@@ -631,6 +933,86 @@ mod tests {
         }
     }
 
+    #[test]
+    fn running_test_snapshots() {
+        let styles = Styles::default();
+        let now = Instant::now();
+
+        for (name, running_test) in running_test_examples(now) {
+            let msg = running_test.message(now, 80, &styles);
+            insta::assert_snapshot!(name, msg);
+        }
+    }
+
+    fn running_test_examples(now: Instant) -> Vec<(&'static str, RunningTest)> {
+        let binary_id = RustBinaryId::new("my-binary");
+        let test_name = "test::my_test".to_string();
+        let start_time = now - Duration::from_secs(125); // 2 minutes 5 seconds ago
+
+        vec![
+            (
+                "running_status",
+                RunningTest {
+                    binary_id: binary_id.clone(),
+                    test_name: test_name.clone(),
+                    status: RunningTestStatus::Running,
+                    start_time,
+                    paused_for: Duration::ZERO,
+                },
+            ),
+            (
+                "slow_status",
+                RunningTest {
+                    binary_id: binary_id.clone(),
+                    test_name: test_name.clone(),
+                    status: RunningTestStatus::Slow,
+                    start_time,
+                    paused_for: Duration::ZERO,
+                },
+            ),
+            (
+                "delay_status",
+                RunningTest {
+                    binary_id: binary_id.clone(),
+                    test_name: test_name.clone(),
+                    status: RunningTestStatus::Delay(Duration::from_secs(130)),
+                    start_time,
+                    paused_for: Duration::ZERO,
+                },
+            ),
+            (
+                "delay_status_underflow",
+                RunningTest {
+                    binary_id: binary_id.clone(),
+                    test_name: test_name.clone(),
+                    status: RunningTestStatus::Delay(Duration::from_secs(124)),
+                    start_time,
+                    paused_for: Duration::ZERO,
+                },
+            ),
+            (
+                "retry_status",
+                RunningTest {
+                    binary_id: binary_id.clone(),
+                    test_name: test_name.clone(),
+                    status: RunningTestStatus::Retry,
+                    start_time,
+                    paused_for: Duration::ZERO,
+                },
+            ),
+            (
+                "with_paused_duration",
+                RunningTest {
+                    binary_id: binary_id.clone(),
+                    test_name: test_name.clone(),
+                    status: RunningTestStatus::Running,
+                    start_time,
+                    paused_for: Duration::from_secs(30),
+                },
+            ),
+        ]
+    }
+
     fn run_stats_test_failure_examples() -> Vec<(&'static str, RunStats)> {
         vec![
             (
