@@ -6,7 +6,7 @@
 //! The main structure in this module is [`TestReporter`].
 
 use super::{
-    FinalStatusLevel, StatusLevel, TestOutputDisplay,
+    FinalStatusLevel, MaxProgressRunning, StatusLevel, TestOutputDisplay,
     displayer::{DisplayReporter, DisplayReporterBuilder, StatusLevels},
 };
 use crate::{
@@ -30,7 +30,7 @@ pub enum ReporterStderr<'a> {
     Terminal,
 
     /// Write output to a buffer.
-    Buffer(&'a mut Vec<u8>),
+    Buffer(&'a mut String),
 }
 
 /// Test reporter builder.
@@ -46,6 +46,7 @@ pub struct ReporterBuilder {
     verbose: bool,
     show_progress: ShowProgress,
     no_output_indent: bool,
+    max_progress_running: MaxProgressRunning,
 }
 
 impl ReporterBuilder {
@@ -105,6 +106,18 @@ impl ReporterBuilder {
         self.no_output_indent = no_output_indent;
         self
     }
+
+    /// Sets the maximum number of running tests to display in the progress bar.
+    ///
+    /// When more tests are running than this limit, only the first N tests are shown
+    /// with a summary line indicating how many more tests are running.
+    pub fn set_max_progress_running(
+        &mut self,
+        max_progress_running: MaxProgressRunning,
+    ) -> &mut Self {
+        self.max_progress_running = max_progress_running;
+        self
+    }
 }
 
 impl ReporterBuilder {
@@ -137,6 +150,7 @@ impl ReporterBuilder {
             no_capture: self.no_capture,
             show_progress: self.show_progress,
             no_output_indent: self.no_output_indent,
+            max_progress_running: self.max_progress_running,
         }
         .build(cargo_configs, output);
 
@@ -161,8 +175,14 @@ pub struct Reporter<'a> {
 
 impl<'a> Reporter<'a> {
     /// Report a test event.
-    pub fn report_event(&mut self, event: TestEvent<'a>) -> Result<(), WriteEventError> {
-        self.write_event(event)
+    pub fn report_event(&mut self, event: ReporterEvent<'a>) -> Result<(), WriteEventError> {
+        match event {
+            ReporterEvent::Tick => {
+                self.tick();
+                Ok(())
+            }
+            ReporterEvent::Test(event) => self.write_event(event),
+        }
     }
 
     /// Mark the reporter done.
@@ -174,8 +194,13 @@ impl<'a> Reporter<'a> {
     // Helper methods
     // ---
 
+    /// Tick the reporter, updating displayed state.
+    fn tick(&mut self) {
+        self.display_reporter.tick();
+    }
+
     /// Report this test event to the given writer.
-    fn write_event(&mut self, event: TestEvent<'a>) -> Result<(), WriteEventError> {
+    fn write_event(&mut self, event: Box<TestEvent<'a>>) -> Result<(), WriteEventError> {
         // TODO: write to all of these even if one of them fails?
         self.display_reporter.write_event(&event)?;
         self.structured_reporter.write_event(&event)?;
