@@ -13,7 +13,9 @@ mod helpers;
 mod imp;
 pub mod structured;
 
-pub use displayer::{FinalStatusLevel, ShowProgress, StatusLevel, TestOutputDisplay};
+pub use displayer::{
+    FinalStatusLevel, MaxProgressRunning, ShowProgress, StatusLevel, TestOutputDisplay,
+};
 pub use error_description::*;
 pub use helpers::highlight_end;
 pub use imp::*;
